'******************************************************************************
'*  io.spin - handle input, output and break requests sent by qz80
'*
'*  (c) Juergen Buchmueller <pullmoll@t-online.de>
'*
'* $Id: io_spin.spin,v 1.1.1.1 2010-04-12 03:51:49 pm Exp $
'******************************************************************************
CON

' Define this to check for contiguous image files
#define	CHECK_CONTIG

' Define this to make 128 byte record writes pending for 1 second
'#define TIMED_FLUSH

	io_cmd_out		=	$01		' I/O emulator command output
	io_cmd_in		=	$02		' I/O emulator command input
	io_cmd_break		=	$03		' I/O emulator command break
	io_cmd_kbd		=	$04		' I/O emulator command read (TRS-80 keyboard)

	dir_none		=	0		' no direction for simh and hdsk ports
	dir_out			=	1		' direction = output for simh and hdsk ports
	dir_in			=	2		' direction = input for simh and hdsk ports

	constat			=	$10		' sio port 1 status port
	condata			=	$11		' sio port 1 data port
	punstat			=	$12		' sio port 2 status port
	pundata			=	$13		' sio port 2 data port

	myname_port		=	$70		' hopefully not used by anything else
	port2_Off		=	$71		' a little CP/M program sends an OUT to port &H71
	Forecolor		=	$72		' set foreground color
	Backcolor		=	$73		' set background color
	keystat			=	$74		' test just keyboard status (constat does this, too)
	keydata			=	$75		' get just the keyboard character

	port2_io		=	$78		' for input and output to port 2
	port2_test		=	$7D		' to test if a byte is ready
	port2_baud		=	$7C		' set baud rate multiple of 1200 eg 1=1200, 16=19200, 96=115200

	hdsk_reset		=	$1		' command to reset controller
	hdsk_read		=	$2		' read command
	hdsk_write		=	$3		' write command
	hdsk_param		=	$4		' get hard disk parameters
	hdsk_port		=	$fd		' control port for simulated hard disk

	simh_setbankselect	=	12		' SIMH command to set memory bank
	simh_reset		=	14		' SIMH reset command
	simh_hasbankedmemory	=	18		' SIMH command to check for banked memory
	simh_stoptimer		=	22		' SIMH stop timer interrupt
	simh_port		=	$fe		' control port for simulated hard disk

	baud			=	115200		' baud rate

VAR
	long	cog
	long	io_command
	byte	name[12+1]
	byte	datetime[20+1]

OBJ
	fdx	: "pcFullDuplexSerial4FC"
	fat	: "fatfs"
#ifdef TriBladeProp
	xmm	: "TriBladeProp_Blade2_204"		' Extended memory manager
#endif TriBladeProp
#ifdef RamBladeProp
	xmm	: "RamBlade_203_002"			' Extended memory manager
#endif RamBladeProp
#ifdef DracBladeProp
	xmm	: "Dracblade"				' Extended memory manager
#endif DracBladeProp
#ifdef HAVE_KBD
	kbd	: "Keyboard"
#endif HAVE_KBD
#ifdef HAVE_TV
	tv80	: "tv80"
#endif
#ifdef HAVE_VGA
	vga	: "VGA_HiRes_Text"
#endif HAVE_VGA

PUB start(pins) : okay | i, dent, sec, size
	stop

	waitcnt(CNT + clkfreq * 2)

	iobuff_ptr := @iobuff

	okay := \fat.start(iobuff_ptr, byte[pins][4], byte[pins][5], byte[pins][6], byte[pins][7])
	if okay < 0
		return -1

	fdx.AddPort(0, byte[pins][0], byte[pins][1], -1, -1, 0, 0, baud)
	if byte[pins][2] <> $ff
		fdx.AddPort(1, byte[pins][2], byte[pins][3], -1, -1, 0, 0, baud)

	okay := fdx.Start
	if okay < 0
		return

	str(string("QZ80 - Quick Z80...",13,10))

#ifdef HAVE_KBD
	if byte[pins][8] == $ff or byte[pins][9] == $ff
		return -1
	okay := \kbd.startx(byte[pins][8], byte[pins][9], %100, 40)
	if okay < 0
		return -1
#endif

#ifdef HAVE_VGA
	if byte[pins][10] == $ff
		return -1
	okay := \vga.start(byte[pins][10], @screen, @colors, @cursor, @vsync)
	if okay > 0
		return -1
	longfill(@colors, %%0020_3300, 64)
	cursor[0] := 0		' cursor x
	cursor[1] := 0		' cursor y
	cursor[2] := %110	' on, blink slow
#endif

	okay := xmm.start
	if okay < 0
		return -1

	okay := fat.mount
	if okay < 0
		failure(string("FAT mount"),okay)
		return

	read_boot_dsk(string("BOOT.DSK"))

	repeat i from 0 to 7
		map_disk(@lba_drive_a+i*4, drive_name[i])

#ifdef HAVE_TV
	tv_command := tv80.get_command
#endif

	okay := cog := COGNEW(io_handler, @io_stack) + 1
	if okay =< 0
		failure(string("I/O"),okay)
		return -1
	success(string("I/O"))

#ifdef HAVE_TV
	if byte[pins][11] == $ff
		return -1
	tv80.start(byte[pins][11], @tv_screen)
#endif

	return 0

PUB stop
'' Stop CPU simulator - frees a cog
	if cog
		cogstop(cog~ - 1)


PUB chr(ch)
'' Transmit a character to the first SIO object
	fdx.tx(0,ch)

PUB str(str_ptr) | i
'' Print a NUL terminated string
	repeat i from 0 to strsize(str_ptr)-1
		chr(byte[str_ptr][i])

PUB crlf
'' Print carriage return and line feed
	chr(13)
	chr(10)

PUB dec(value) | _i
'' Print a decimal number
	if value < 0
		-value
		chr("-")

	_i := 1_000_000_000

	repeat 10
		if value => _i
			chr(value / _i + "0")
			value //= _i
			result~~
		elseif result or _i == 1
			chr("0")
		_i /= 10

PUB dec_size(value)
'' Print a (short) file or image size value
	if value > 1048576
		dec(value/1048576)
		chr(".")
		dec(10*(value // 1048576) / 1048576)
		str(string("MB"))
	elseif value > 1024
		dec(value/1024)
		chr(".")
		dec(10*(value // 1024) / 1024)
		str(string("KB"))
	else
		dec(value)

PUB hex(value, digits)
'' Print a hexadecimal number
	value <<= (8 - digits) << 2
	repeat digits
		chr(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))

PUB bin(value, digits)
'' Print a binary number
	value <<= 32 - digits
	repeat digits
		chr((value <-= 1) & 1 + "0")

PRI failure(who, val)
	str(who)
	str(string(" failed: "))
	dec(val)
	crlf

PRI success(who) | i, n, n1, n2, n3, n4, n5, n6, n7
	str(who)
	str(string(" initialized, "))
	dec(free_cogs)
	str(string(" cogs free.", 13, 10))

PUB free_cogs : free | i, cogs[8], jmp_0
	jmp_0 := %010111_0001_1111_000000000_000000000
	repeat while (cogs[free] := cognew(@jmp_0, 0)) => 0 
		free++
	if free
		repeat i from 0 to free - 1
			cogstop(cogs[i])
	return free

PRI read_boot_dsk(fname_ptr) | dent, size, sec
'' Read the BOOT.DSK file and copy it to Z80 memory $ff00..$ffff. Patch $0000 to jump there.
	dent := fat.find_file(fname_ptr)
	if dent == 0
		failure(fname_ptr,dent)
		return -1
	success(fname_ptr)
	size := fat.get_dent_size(dent)
	sec := fat.get_dent_first_sector(dent)
	fat.get_dent_name(dent, @name)
	fat.get_dent_date_time(dent, @datetime)
	str(@name)
	str(string(": sector "))
	dec(sec)
	str(string(9,"size "))
	dec_size(size)
	chr(9)
	str(@datetime)
	crlf
	fat.read_sector(sec)
	xmm.DoCmd("W", iobuff_ptr, $FF00, 256)
	long[iobuff_ptr] := $00FF00C3
	xmm.DoCmd("W", iobuff_ptr, $0000, 4)


PRI map_disk(lbaptr, fname_ptr) : okay | dent, size, clus, exp, err, sec
'' Map a FAT file to a disk, setting the first sector and (optionally) checking its continuity
	dent := fat.find_file(fname_ptr)
	if dent < 0
		failure(fname_ptr, dent)
		return -1
	sec := fat.get_dent_first_sector(dent)
	size := fat.get_dent_size(dent)
	long[lbaptr] := sec
	fat.get_dent_name(dent, @name)
	fat.get_dent_date_time(dent, @datetime)
	str(@name)
	str(string(9,"sector "))
	hex(sec,8)
	str(string(9,"size "))
	dec_size(size)
	chr(9)
	str(@datetime)
#ifdef CHECK_CONTIG
	clus := fat.get_dent_cluster(dent)
	exp := clus + 1
	err := 0
	repeat while (clus := fat.fat_next(clus)) => 0
		if clus <> exp
			err++
			quit
		exp := exp + 1
	if err
		str(string(" not contiguous!"))
	else
		str(string(" contiguous - okay."))
#endif
	crlf

PUB command_addr
'' Return the address of the 3 longs block
	return @io_command

PRI io_handler | cmd, port
'' I/O command handler running in a separate cog
	repeat
		repeat while (io_command & $ff000000) == 0
		cmd := io_command >> 24
		port := io_command >> 8
		case cmd
			io_cmd_out:
				io_out(port & $ff, io_command & $ff)
			io_cmd_in:
				io_command := (io_command & !$ff) | io_in(port & $ff)
			io_cmd_break:
				io_break(port)
		io_command &= $00ffffff

PRI io_out(port, data)
	case port	
		constat:	' sio port 1 status port
			' ???
		condata:	' sio port 1 data port
			fdx.tx(0,data)
		punstat:	' sio port 2 status port
			' ???
		pundata:	' sio port 2 data port
			fdx.tx(1,data)
		myname_port:	' hopefully not used by anything else
			' ???
		port2_Off:	' a little CP/M program sends an OUT to port &H71
			' ???
		Forecolor:	' set foreground color
			' ???
		Backcolor:	' set background color
			' ???
		keystat:	' test just keyboard status (constat does this, too)
			' ???
		keydata:	' get just the keyboard character
			' ???
		port2_io:	' for input and output to port 2
			fdx.tx(1,data)
		port2_test:	' to test if a byte is ready
			' ???
		port2_baud:	' set baud rate multiple of 1200 eg 1=1200, 16=19200, 96=115200
			' ???
		hdsk_port:	' control port for simulated hard disk
			if direction == dir_out and hdsk_params_cnt > 0
				byte[hdsk_params_ptr++] := data
				hdsk_params_cnt--
				if hdsk_params_cnt == 0
					direction := dir_none
			else
				hdsk_command := data
				case hdsk_command
					hdsk_reset:	' command to reset controller
						hdsk_params_ptr := 0
						hdsk_params_cnt := 0
						hdsk_status := 0
					hdsk_read:	' read command
						direction := dir_out
						hdsk_params_ptr := @hdsk_params
						hdsk_params_cnt := @hdsk_params_end - @hdsk_params
					hdsk_write:	' write command
						direction := dir_out
						hdsk_params_ptr := @hdsk_params
						hdsk_params_cnt := @hdsk_params_end - @hdsk_params
					hdsk_param:	' get hard disk parameters
						direction := dir_in
						hdsk_params_ptr := @hdsk_dpb
						hdsk_params_cnt := @hdsk_dpb_end - @hdsk_dpb
					other:
						hdsk_command := 0
						hdsk_status := 1
				
		simh_port:	' control port for SIMH compatibility
	return

PRI io_in(port) | data, kbstat, sio0stat, sio1stat
	case port	
		constat:	' sio port 1 status port
#ifdef HAVE_KBD
			kbstat := kbd.gotKey
#endif
			sio0stat := fdx.rxavail(0)
			sio1stat := fdx.rxavail(1)
			if kbstat or sio0stat or sio1stat
				data := %00000011
			else
				data := %00000010
		condata:	' sio port 1 data port
			sio0stat := fdx.rxavail(0)
			if sio0stat
				data := fdx.rx(0)
		punstat:	' sio port 2 status port
			sio1stat := fdx.rxavail(1)
		pundata:	' sio port 2 data port
			sio1stat := fdx.rxavail(1)
			if sio1stat
				data := fdx.rx(1)
		myname_port:	' hopefully not used by anything else
			data := 0
		port2_Off:	' a little CP/M program sends an OUT to port &H71
			data := 0
		Forecolor:	' set foreground color
			data := 0
		Backcolor:	' set background color
			data := 0
		keystat:	' test just keyboard status (constat does this, too)
#ifdef HAVE_KBD
			kbstat := kbd.gotKey
			if kbstat
				data := %00000011
			else
				data := %00000010
#else
			data := %00000010
#endif
		keydata:	' get just the keyboard character
			kbstat := kbd.gotKey
			if kbstat
				data := kbd.getKey
		port2_io:	' for input and output to port 2
			sio1stat := fdx.rxavail(1)
			if sio1stat
				data := fdx.rx(1)
		port2_test:	' to test if a byte is ready
			sio1stat := fdx.rxavail(1)
			if sio1stat
				data := %00100001
			else
				data := %00100001
		port2_baud:	' set baud rate multiple of 1200 eg 1=1200, 16=19200, 96=115200
			data := 0
		hdsk_port:	' control port for simulated hard disk
			if direction == dir_in and hdsk_params_cnt > 0
				data := byte[hdsk_params_ptr++]
				hdsk_params_cnt--
				if hdsk_params_cnt == 0
					direction := dir_none
					hdsk_params_ptr := 0
					hdsk_command := 0
			else
				case hdsk_command
					hdsk_reset:	' command to reset controller
						hdsk_status := 0
					hdsk_read:	' read command
						hdsk_status := hdsk_read_sector
					hdsk_write:	' write command
						hdsk_status := hdsk_write_sector
					hdsk_param:	' get hard disk parameters
						hdsk_status := 0
				data := hdsk_status
		simh_port:	' control port for SIMH compatibility
			if direction == dir_in and simh_params_cnt > 0
				data := byte[simh_params_ptr++]
				simh_params_cnt--
				if simh_params_cnt == 0
					direction := dir_none
					simh_params_ptr := 0
					simh_command := 0
			else
				case simh_command
					simh_setbankselect:	' SIMH command to set memory bank
						simh_status := 0
					simh_reset:		' SIMH reset command
						simh_status := 0
					simh_hasbankedmemory:	' SIMH command to check for banked memory
						simh_status := 0
					simh_stoptimer:		' SIMH stop timer interrupt
						simh_status := 0
				data := simh_status
	return data

PRI io_break(base)
	str(msg_break_bc)
	hex(word[base][ 0],4)
	str(msg_break_de)
	hex(word[base][ 1],4)
	str(msg_break_hl)
	hex(word[base][ 2],4)
	str(msg_break_af)
	hex(word[base][ 3],4)
	str(msg_break_bc2)
	hex(word[base][ 4],4)
	str(msg_break_de2)
	hex(word[base][ 5],4)
	str(msg_break_hl2)
	hex(word[base][ 6],4)
	str(msg_break_af2)
	hex(word[base][ 7],4)
	str(msg_break_r)
	hex((byte[base][16] & $7f) | byte[base][17],4)
	str(msg_break_iff1)
	hex(byte[base][18] & 1, 1)
	str(msg_break_iff2)
	hex((byte[base][18] >> 1) & 1, 1)
	str(msg_break_im)
	hex((byte[base][18] >> 2) & 3, 1)
	str(msg_break_ix)
	hex(word[base][10],4)
	str(msg_break_iy)
	hex(word[base][11],4)
	str(msg_break_sp)
	hex(word[base][12],4)
	str(msg_break_pc)
	hex(word[base][13],4)
	crlf

PRI hdsk_read_sector : status | record, offset, lba
	if hdsk_drive => 8
		return 2
	record := hdsk_track * hdsk_spt + hdsk_sector
	offset := (record & 3) << 7
	lba := lba_drive_a[hdsk_drive] + (record >> 2)
	if lba <> lba_iobuff
		status := fat.read_sector(lba)
		if status < 0
			return
		lba_iobuff := lba
	xmm.DoCmd("W", iobuff_ptr + offset, hdsk_dma, 128)
	return 0

PRI hdsk_write_sector : status | record, offset, lba
	if hdsk_drive => 8
		return 2
	record := hdsk_track * hdsk_spt + hdsk_sector
	offset := (record & 3) << 7
	lba := lba_drive_a[hdsk_drive] + (record >> 2)
	if lba <> lba_iobuff
		status := fat.read_sector(lba)
		if status < 0
			return
		lba_iobuff := lba
	xmm.DoCmd("R", iobuff_ptr + offset, hdsk_dma, 128)
	status := fat.write_sector(lba)
	return 0

DAT
io_stack	long	0[64]
iobuff		long	0[2*512/4]		' I/O buffer (need 2 because of FAT12)

iobuff_ptr	long	0
lba_iobuff	long	-1			' LBA that is currently in iobuff

lba_drive_a	long	0,0,0,0,0,0,0,0		' first sector of drive A:, B: ... H:

simh_params	byte	0,0,0,0,0,0		' parameters of command
simh_status	word	0
simh_command	long	0
simh_params_ptr	long	0
simh_params_cnt	long	0

hdsk_params	byte
hdsk_drive	byte	0
hdsk_sector	byte	0
hdsk_track	word	0
hdsk_dma	word	0
hdsk_params_end	byte
hdsk_status	word	0
hdsk_command	long	0
hdsk_params_ptr	long	0
hdsk_params_cnt	long	0

'Hard Disk Parameter Blocks
'Each DPB is 17 bytes as defined for DPBs in CP/M followed by 2 bytes sector size as used by SIMH
hdsk_dpb	byte			' 8MByte Default SIMH Altair HDSK params
hdsk_spt	word	$20		' sectors per track
hdsk_bsh	byte	$05		' data allocation Block SHift factor
hdsk_blm	byte	$1F		' data allocation block mask
hdsk_exm	byte	$01		' EXtent Mask
hdsk_dsm_lo	byte	$F9		' maximum data block number (low_byte)
hdsk_dsm_hi	byte	$07		' maximum data block number (high_byte)
hdsk_drm_lo	byte	$FF		' total number of directory entries (low byte)
hdsk_drm_hi	byte	$03		' total number of directory entries (high byte)
hdsk_al0	byte	$FF		' determine reserved directory blocks
hdsk_al1	byte	$00		' determine reserved directory blocks
hdsk_cks_lo	byte	$00		' size of directory ChecK vector (low byte)
hdsk_cks_hi	byte	$00		' size of directory ChecK vector (high byte)
hdsk_off_lo	byte	$06		' number of reserved tracks (offset) (low byte)
hdsk_off_hi	byte	$00		' number of reserved tracks (offset) (high byte)
hdsk_psh	byte	$00		' Physical record SHift factor, CP/M 3
hdsk_phm	byte	$00		' Physical record Mask, CP/M 3
hdsk_ss_lo	byte	$80		' Sector Size (low byte)
hdsk_ss_hi	byte	$00		' Sector Size (high byte)
hdsk_dpb_end	byte	0		' padding to 20 bytes

#ifdef HAVE_VGA
vsync		long	0
screen		long	0[80*40/4]
colors		long	0[64]
cursor		byte	0[6]
#endif HAVE_VGA

#ifdef HAVE_TV
tv_screen	long	0[80*25/4]
tv_command	long	0
#endif HAVE_TV

direction	byte	0

msg_break_bc	byte	"BC:",0
msg_break_de	byte	" DE:",0
msg_break_hl	byte	" HL:",0
msg_break_af	byte	" AF:",0
msg_break_bc2	byte	" BC'",0
msg_break_de2	byte	" DE'",0
msg_break_hl2	byte	" HL'",0
msg_break_af2	byte	" AF'",0
msg_break_r	byte	13,10, "R:",0
msg_break_iff1	byte	" IFF1:",0
msg_break_iff2	byte	" IFF2:",0
msg_break_im	byte	" IM:",0
msg_break_i	byte	" I:",0
msg_break_ix	byte	" IX:",0
msg_break_iy	byte	" IY:",0
msg_break_sp	byte	" SP:",0
msg_break_pc	byte	" PC:",0
msg_break_data	byte	" = ",0

drive_a		byte	"A.DSK",0
drive_b		byte	"B.DSK",0
drive_c		byte	"C.DSK",0
drive_d		byte	"D.DSK",0
drive_e		byte	"E.DSK",0
drive_f		byte	"F.DSK",0
drive_g		byte	"G.DSK",0
drive_h		byte	"H.DSK",0
drive_name	long	@@@drive_a, @@@drive_b, @@@drive_c, @@@drive_d
		long	@@@drive_e, @@@drive_f, @@@drive_g, @@@drive_h
