'******************************************************************************
'*  tv80.spin - TV text mode with 80x20 characters
'*
'*  This is based on ideas and code from Debug_1pinTV by "Cluso99"
'*
'*  (c) Juergen Buchmueller <pullmoll@t-online.de>
'*
'* $Id: tv80.spin,v 1.1.1.1 2010-04-12 03:51:49 pm Exp $
'******************************************************************************
CON
	_xinfreq		= 5_000_000		' Quartz is 5MHz
	_clkmode		= xtal1 + pll16x	' System clock is 80MHz

	FONT_W			= 6
	FONT_H			= 9

#ifndef NTSC
#ifndef PAL
#define PAL
#endif  PAL
#endif  NTSC

#ifdef NTSC
'' NTSC 80x25 using a 6x9 font
ocols			=	80			' number of columns
orows			=	25			' number of rows
ocolsfrq		=	22_478			' Hz per active pixel
oequal1			=	6			' number of equalization pulses
oserr			=	6			' number of serration pulses
oequal2			=	6			' number of equalization pulses
oblank1			=	25			' number of blank lines
oactive			=	orows*FONT_H		' number of active lines
oblank2			=	1			' number of blank lines
ohalf			=	343			' vscl half line
obackp			=	94			' vscl back porch (sync to active)
ofrontp			=	62			' vscl front porch (active to sync)
oequal			=	25			' vscl equalization pulse (sync/2)
osync			=	50			' vscl sync pulse
osynch			=	293			' vscl half-sync
oequalh			=	318			' vscl half-equal
ovsclch			=	4*FONT_W		' vscl character
ictra			=	12			' CTRA with PLLdiv
ifrqa			=	231701456		' required FRQA value (5MHz*16) (xfrqa replaced by start routine)
ifrqb			=	$4924_0000		' 40/140 << 32 = 1,227,096,064

ifrqa_80MHz		=	289626816		' required FRQA value (5MHz*16)
ifrqa_96MHz		=	241355680		' required FRQA value (6MHz*16)
ifrqa_100MHz		=	231701456		' required FRQA value (6.25MHz*16)
ifrqa_104MHz		=	222789856		' required FRQA value (6.5MHz*16)
ifrqa_108MHz		=	214538384		' required FRQA value (13.5MHz*8)
#endif

'--------------------------------------------------------------------------------------------------
#ifdef PAL
'' PAL 80x25 using a 6x9 font
ocols			=	80			' number of columns
orows			=	25			' number of rows
ocolsfrq		=	22_321			' Hz per active pixel
oequal1			=	4			' number of equalization pulses
oserr			=	5			' number of serration pulses
oequal2			=	5			' number of equalization pulses
oblank1			=	59			' number of blank lines
oactive			=	orows*FONT_H		' number of active lines
oblank2			=	28			' number of blank lines
ohalf			=	342			' vscl half line
obackp			=	108			' vscl back porch (sync to active)
ofrontp			=	39			' vscl front porch (active to sync)
oequal			=	24			' vscl equalization pulse (sync/2)
osync			=	49			' vscl sync pulse
osynch			=	293			' vscl half-sync
oequalh			=	318			' vscl half-equal
ovsclch			=	4*FONT_W		' vscl character
ictra			=	12			' CTRA with PLLdiv
ifrqa			=	287604888		' required FRQA value (5MHz*16) (xfrqa replaced by start routine)
ifrqb			=	$4924_0000		' 40/140 << 32 = 1,227,096,064

ifrqa_80MHz		=	287604888		' required FRQA value (5MHz*16)
ifrqa_96MHz		=	239669904		' required FRQA value (6MHz*16)
ifrqa_100MHz		=	230083104		' required FRQA value (6.25MHz*16)
ifrqa_104MHz		=	221233760		' required FRQA value (6.5MHz*16)
ifrqa_108MHz		=	213039920		' required FRQA value (13.5MHz*8)
#endif
'--------------------------------------------------------------------------------------------------

flash			=	1<<4

deccm			=	%00000001		' DEC cursor mode (0: off, 1: on)
decim			=	%00000010		' DEC insert mode
decom			=	%00000100		' DEC origin mode
deccr			=	%00001000		' DEC send CRLF or LF (0: LF, 1: CRLF)
decck			=	%00010000		' DEC send cursor keys
decawm			=	%00100000		' DEC auto wrap mode
decarm			=	%01000000		' DEC auto repeat mode
decrm			=	%10000000		' DEC report mouse

attr_highlight		=	%00000001
attr_underline		=	%00000010
attr_blinking		=	%00000100
attr_inverse		=	%00001000
attr_fg			=	5			' 3 bits
attr_bg			=	8			' 3 bits

esc_mode_off		=	0
esc_mode_esc		=	1
esc_mode_csi		=	2

VAR
		long	cog

PUB start(params) : okay
' bitmap is the address of the bitmap buffer * 32 + the tvPin number

	stop
	Setforxtalfreq					'set ifrqa in hub for the current clkfreq before cognew 
	okay := cog := COGNEW(@entry, params) + 1

PUB stop
	if cog
		COGSTOP(cog - 1)
		cog := 0

PRI Setforxtalfreq
'' Set the frqa value depending on the actual clkfreq
	Case clkfreq
		 80_000_000: xfrqa := ifrqa_80MHz	' 5.00 MHz
		 96_000_000: xfrqa := ifrqa_96MHz	' 6.00 MHz
		100_000_000: xfrqa := ifrqa_100MHz	' 6.25 MHz
		104_000_000: xfrqa := ifrqa_104MHz	' 6.50 MHz
		108_000_000: xfrqa := ifrqa_108MHz	'13.50 MHz

DAT
		org	0
entry
		mov	t1, PAR
		rdlong	pin_no, t1
		add	t1, #4
		rdlong	screen_ptr, t1
		add	screen_end, screen_ptr

		mov	FRQB, xfrqb			' default = 40/140

		movi	CTRA, #ictra
		mov	FRQA, xfrqa

		movs	CTRB, pin_no			' set pin
		mov	count, #1
		shl	count, pin_no
		mov	DIRA, count			' set pin mask
		mov	count, pin_no
		shr	count, #3
		movd	VCFG, count			' set VGroup
		mov	count, #1
		and	pin_no, #7
		shl	count, pin_no
		movs	VCFG, count			' set VPins
		movi	VCFG, #%0_01_0_0_0_000		' VGA mode, 1 bit per pixel

		movd	VSCL, #(1 << 3)			' 1 pixel

'**************************************************************************************************
frame		add	framectr, #1			' inc frame counter
		mov	rows, #oequal1
		call	#equalizing
'--------------------------------------------------------------------------------------------------
		mov	rows, #oserr			' serration pulses
		call	#serration
'--------------------------------------------------------------------------------------------------
		mov	rows, #oequal2			' equalization pulses
		call	#equalizing
'==================================================================================================
		mov	rows, #oblank1
		call	#doblank
'==================================================================================================
'display active lines
		mov	fontindex, font_ptr
		mov	scrptr0, screen_ptr		' initialize bitmap pointer
		mov	rows, #oactive
doactive
		movs	vscl, #osync			' 4.7uSec @ -40 IRE
		waitvid xff00, #0
		movi	CTRB, #0			' turn off blank
		mov	count, #ocols/4			' characters per line / 4, since we do 4 at a time
		movs	vscl, #obackp			' back porch
		waitvid xff00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		movs	vscl, #ovsclch			' 1 PLLA per pixel, 24 PLLA per frame
		mov	scrptr, scrptr0			' start at character row address (again)
'--------------------------------------------------------------------------------------------------
:active
		rdlong	ch2, scrptr			' get 4 characters
		add	scrptr, #4

		mov	ch1, ch2
		and	ch1, #%01111111
		add	ch1, fontindex			' + fontindex = address of font row
		rdbyte	bits0, ch1			' read to bits0

		ror	ch2, #8
		mov	ch1, ch2
		and	ch1, #%01111111
		add	ch1, fontindex			' + fontindex = address of font row
		rdbyte	bits1, ch1			' read to bits1
		shl	bits1, #1*6			' to bits 11..6

		ror	ch2, #8
		mov	ch1, ch2
		and	ch1, #%01111111
		add	ch1, fontindex			' + fontindex = address of font row
		rdbyte	bits2, ch1			' read to bits2
		shl	bits2, #2*6			' to bits 17..12

		ror	ch2, #8
		and	ch2, #%01111111
		add	ch2, fontindex			' + fontindex = address of font row
		rdbyte	bits3, ch2			' read to bits3
		shl	bits3, #3*6			' to bits 23..18

		or	bits0, bits1
		or	bits0, bits2
		or	bits0, bits3

		waitvid	xff00, bits0			' output 24 bits to screen
		djnz	count, #:active			' do entire line
'--------------------------------------------------------------------------------------------------

		add	fontindex, #128
		cmp	fontindex, font_end	WZ
	if_z	mov	fontindex, font_ptr
	if_z	add	scrptr0, #ocols

		movs	vscl, #ofrontp			' front porch
		waitvid xff00, #0			' output blank
		djnz	rows, #doactive			' next row
'==================================================================================================
		mov	rows, #oblank2
		call	#doblank
'==================================================================================================
		jmp	#frame

doblank
		movs	vscl, #osync			' 4.7uSec @ -40 IRE
		waitvid xff00, #0
		movi	CTRB, #0			' turn off blank
		movs	vscl, #osynch			' blank line
		waitvid xff00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		movs	vscl, #ohalf			' half line ???
		waitvid xff00, #0
		djnz	rows, #doblank
doblank_ret
		ret

equalizing
		movs	vscl, #oequal			' equalizing pulse (short)
		waitvid xff00, #0
		movi	CTRB, #0			' turn off blank
		movs	vscl, #oequalh			' equalizing pulse (long)
		waitvid xff00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		djnz	rows, #equalizing
equalizing_ret
		ret

serration
		movs	vscl, #osynch			' serration pulse (long)
		waitvid xff00, #0		
		movi	CTRB, #0			' turn off blank
		movs	vscl, #osync			' serration pulse (short)
		waitvid xff00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		djnz	rows, #serration
serration_ret
		ret
'--------------------------------------------------------------------------------------------------
'constants > $1FF
xfrqa		long	ifrqa
xfrqb		long	ifrqb

xff00		long	$ff00			' white / black
low_word	long	$ffff
x20202020	long	$20202020
x7f		long	$7f
font_ptr	long	@@@font
font_end	long	@@@font + 128*FONT_H
pin_no		long	0
screen_ptr	long	0-0
screen_end	long	ocols*orows*2
framectr	long	0

t1		res	1
ch1		res	1
ch2		res	1
bits0		res	1			' 1st bit mask
bits1		res	1			' 2nd bit mask
bits2		res	1			' 3rd bit mask
bits3		res	1			' 4th bit mask
fontindex	res	1
scrptr0		res	1			' pointer to screen data per character row
scrptr		res	1			' pointer to screen data per scanline
count		res	1			' all purpose counter
rows		res	1			' row counter
'**************************************************************************************************
		fit	$1f0

'**************************************************************************************************
font
				' scanrow 0
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%000100	' ..#...
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%100000	' .....#
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%010100	' ..#.#.
		byte	%010100	' ..#.#.
		byte	%001000	' ...#..
		byte	%000110	' .##...
		byte	%000100	' ..#...
		byte	%001100	' ..##..
		byte	%010000	' ....#.
		byte	%000100	' ..#...
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%010000	' ....#.
		byte	%111110	' .#####
		byte	%011000	' ...##.
		byte	%111110	' .#####
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%011110	' .####.
		byte	%011100	' ..###.
		byte	%011110	' .####.
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%111100	' ..####
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%011110	' .####.
		byte	%011100	' ..###.
		byte	%011110	' .####.
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%111110	' .#####
		byte	%111100	' ..####
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%000000	' ......
		byte	%011000	' ...##.
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%100000	' .....#
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%001000	' ...#..
		byte	%000100	' ..#...
		byte	%000100	' ..#...
		byte	%111111	' ######

				' scanrow 1
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%000100	' ..#...
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%101000	' ...#.#
		byte	%010100	' ..#.#.
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%010100	' ..#.#.
		byte	%010100	' ..#.#.
		byte	%111100	' ..####
		byte	%100110	' .##..#
		byte	%001010	' .#.#..
		byte	%001100	' ..##..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%001100	' ..##..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%011000	' ...##.
		byte	%000010	' .#....
		byte	%000100	' ..#...
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%010100	' ..#.#.
		byte	%100100	' ..#..#
		byte	%100010	' .#...#
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%010010	' .#..#.
		byte	%000010	' .#....
		byte	%110110	' .##.##
		byte	%100110	' .##..#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100000	' .....#
		byte	%001100	' ..##..
		byte	%000010	' .#....
		byte	%001111	' ####..
		byte	%001010	' .#.#..
		byte	%000000	' ......
		byte	%011000	' ...##.
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%100000	' .....#
		byte	%000000	' ......
		byte	%101000	' ...#.#
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%111111	' ######

				' scanrow 2
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%010000	' ....#.
		byte	%110110	' .##.##
		byte	%010000	' ....#.
		byte	%100010	' .#...#
		byte	%011110	' .####.
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%000100	' ..#...
		byte	%110110	' .##.##
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%010000	' ....#.
		byte	%010100	' ..#.#.
		byte	%100000	' .....#
		byte	%010100	' ..#.#.
		byte	%011100	' ..###.
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%010100	' ..#.#.
		byte	%111110	' .#####
		byte	%001010	' .#.#..
		byte	%010000	' ....#.
		byte	%001010	' .#.#..
		byte	%000100	' ..#...
		byte	%000100	' ..#...
		byte	%010000	' ....#.
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%110010	' .#..##
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%100000	' .....#
		byte	%010100	' ..#.#.
		byte	%011110	' .####.
		byte	%000010	' .#....
		byte	%010000	' ....#.
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001100	' ..##..
		byte	%001100	' ..##..
		byte	%000100	' ..#...
		byte	%111110	' .#####
		byte	%010000	' ....#.
		byte	%100000	' .....#
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%001010	' .#.#..
		byte	%000010	' .#....
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%010100	' ..#.#.
		byte	%010100	' ..#.#.
		byte	%010000	' ....#.
		byte	%001100	' ..##..
		byte	%000100	' ..#...
		byte	%001100	' ..##..
		byte	%010001	' #...#.
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%011010	' .#.##.
		byte	%011100	' ..###.
		byte	%101100	' ..##.#
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%101100	' ..##.#
		byte	%011010	' .#.##.
		byte	%001100	' ..##..
		byte	%100000	' .....#
		byte	%010010	' .#..#.
		byte	%001000	' ...#..
		byte	%010110	' .##.#.
		byte	%011010	' .#.##.
		byte	%011100	' ..###.
		byte	%011010	' .#.##.
		byte	%101100	' ..##.#
		byte	%011010	' .#.##.
		byte	%111100	' ..####
		byte	%111110	' .#####
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%010000	' ....#.
		byte	%111111	' ######

				' scanrow 3
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%111100	' ..####
		byte	%101010	' .#.#.#
		byte	%001010	' .#.#..
		byte	%111110	' .#####
		byte	%100100	' ..#..#
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%111110	' .#####
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%111110	' .#####
		byte	%111010	' .#.###
		byte	%111010	' .#.###
		byte	%101110	' .###.#
		byte	%101110	' .###.#
		byte	%101010	' .#.#.#
		byte	%010100	' ..#.#.
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%000100	' ..#...
		byte	%111110	' .#####
		byte	%101110	' .###.#
		byte	%101110	' .###.#
		byte	%111010	' .#.###
		byte	%111010	' .#.###
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%010100	' ..#.#.
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%000100	' ..#...
		byte	%000010	' .#....
		byte	%000100	' ..#...
		byte	%010000	' ....#.
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%011000	' ...##.
		byte	%010010	' .#..#.
		byte	%100000	' .....#
		byte	%011110	' .####.
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%111100	' ..####
		byte	%000000	' ......
		byte	%001100	' ..##..
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%100000	' .....#
		byte	%010000	' ....#.
		byte	%101100	' ..##.#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%000010	' .#....
		byte	%100100	' ..#..#
		byte	%001110	' .###..
		byte	%001110	' .###..
		byte	%110010	' .#..##
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%000110	' .##...
		byte	%000010	' .#....
		byte	%101010	' .#.#.#
		byte	%110010	' .#..##
		byte	%100010	' .#...#
		byte	%011110	' .####.
		byte	%100010	' .#...#
		byte	%011110	' .####.
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%010100	' ..#.#.
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001100	' ..##..
		byte	%001000	' ...#..
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%100000	' .....#
		byte	%100110	' .##..#
		byte	%100010	' .#...#
		byte	%110010	' .#..##
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%110010	' .#..##
		byte	%100110	' .##..#
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001010	' .#.#..
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%100110	' .##..#
		byte	%100010	' .#...#
		byte	%100110	' .##..#
		byte	%110010	' .#..##
		byte	%100110	' .##..#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%010100	' ..#.#.
		byte	%100010	' .#...#
		byte	%010000	' ....#.
		byte	%000100	' ..#...
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%111111	' ######

				' scanrow 4
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%001000	' ...#..
		byte	%110110	' .##.##
		byte	%000110	' .##...
		byte	%010100	' ..#.#.
		byte	%101000	' ...#.#
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%101010	' .#.#.#
		byte	%000100	' ..#...
		byte	%110110	' .##.##
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%000110	' .##...
		byte	%010100	' ..#.#.
		byte	%100000	' .....#
		byte	%010100	' ..#.#.
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%111110	' .#####
		byte	%101000	' ...#.#
		byte	%000100	' ..#...
		byte	%101010	' .#.#.#
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%010000	' ....#.
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%100110	' .##..#
		byte	%001000	' ...#..
		byte	%000010	' .#....
		byte	%100000	' .....#
		byte	%111110	' .#####
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%000100	' ..#...
		byte	%100010	' .#...#
		byte	%100000	' .....#
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%111110	' .#####
		byte	%010000	' ....#.
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%111110	' .#####
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%001010	' .#.#..
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%101010	' .#.#.#
		byte	%001010	' .#.#..
		byte	%100000	' .....#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%010100	' ..#.#.
		byte	%101010	' .#.#.#
		byte	%010100	' ..#.#.
		byte	%001000	' ...#..
		byte	%000100	' ..#...
		byte	%001100	' ..##..
		byte	%010000	' ....#.
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%111100	' ..####
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%110010	' .#..##
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%000110	' .##...
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%111111	' ######

				' scanrow 5
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%010000	' ....#.
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%010100	' ..#.#.
		byte	%100000	' .....#
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%010100	' ..#.#.
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%101010	' .#.#.#
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%010100	' ..#.#.
		byte	%011110	' .####.
		byte	%110010	' .#..##
		byte	%010010	' .#..#.
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%001000	' ...#..
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%001100	' ..##..
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%010000	' ....#.
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%010000	' ....#.
		byte	%001100	' ..##..
		byte	%001100	' ..##..
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100100	' ..#..#
		byte	%100010	' .#...#
		byte	%100100	' ..#..#
		byte	%000010	' .#....
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%010010	' .#..#.
		byte	%000010	' .#....
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%000010	' .#....
		byte	%010010	' .#..#.
		byte	%010010	' .#..#.
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%110110	' .##.##
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%000010	' .#....
		byte	%001100	' ..##..
		byte	%100000	' .....#
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%100010	' .#...#
		byte	%100110	' .##..#
		byte	%100010	' .#...#
		byte	%110010	' .#..##
		byte	%000010	' .#....
		byte	%001000	' ...#..
		byte	%101100	' ..##.#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%001010	' .#.#..
		byte	%001000	' ...#..
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%100110	' .##..#
		byte	%110010	' .#..##
		byte	%000010	' .#....
		byte	%100000	' .....#
		byte	%101000	' ...#.#
		byte	%110010	' .#..##
		byte	%010100	' ..#.#.
		byte	%101010	' .#.#.#
		byte	%010100	' ..#.#.
		byte	%111100	' ..####
		byte	%000100	' ..#...
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%111111	' ######

				' scanrow 6
		byte	%111110	' .#####
		byte	%000010	' .#....
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%100000	' .....#
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%110110	' .##.##
		byte	%100000	' .....#
		byte	%000000	' ......
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%110110	' .##.##
		byte	%100000	' .....#
		byte	%111110	' .#####
		byte	%001000	' ...#..
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%010100	' ..#.#.
		byte	%001000	' ...#..
		byte	%110000	' ....##
		byte	%101100	' ..##.#
		byte	%000000	' ......
		byte	%010000	' ....#.
		byte	%000100	' ..#...
		byte	%001000	' ...#..
		byte	%000000	' ......
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%111110	' .#####
		byte	%011100	' ..###.
		byte	%010000	' ....#.
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%000010	' .#....
		byte	%011100	' ..###.
		byte	%001100	' ..##..
		byte	%000000	' ......
		byte	%001100	' ..##..
		byte	%010000	' ....#.
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%100010	' .#...#
		byte	%011110	' .####.
		byte	%011100	' ..###.
		byte	%011110	' .####.
		byte	%111110	' .#####
		byte	%000010	' .#....
		byte	%111100	' ..####
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%011100	' ..###.
		byte	%100010	' .#...#
		byte	%111110	' .#####
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%000010	' .#....
		byte	%101100	' ..##.#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%100010	' .#...#
		byte	%100010	' .#...#
		byte	%001000	' ...#..
		byte	%111110	' .#####
		byte	%111100	' ..####
		byte	%000000	' ......
		byte	%001111	' ####..
		byte	%000000	' ......
		byte	%111110	' .#####
		byte	%000000	' ......
		byte	%111100	' ..####
		byte	%011010	' .#.##.
		byte	%011100	' ..###.
		byte	%101100	' ..##.#
		byte	%011100	' ..###.
		byte	%001000	' ...#..
		byte	%100000	' .....#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%100000	' .....#
		byte	%010010	' .#..#.
		byte	%011100	' ..###.
		byte	%101010	' .#.#.#
		byte	%100010	' .#...#
		byte	%011100	' ..###.
		byte	%011010	' .#.##.
		byte	%101100	' ..##.#
		byte	%000010	' .#....
		byte	%011110	' .####.
		byte	%010000	' ....#.
		byte	%101100	' ..##.#
		byte	%001000	' ...#..
		byte	%010100	' ..#.#.
		byte	%100010	' .#...#
		byte	%100000	' .....#
		byte	%111110	' .#####
		byte	%010000	' ....#.
		byte	%001000	' ...#..
		byte	%000100	' ..#...
		byte	%000000	' ......
		byte	%111111	' ######

				' scanrow 7
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000100	' ..#...
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%100010	' .#...#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%100010	' .#...#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%100000	' .....#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%100010	' .#...#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%111111	' ######

				' scanrow 8
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000010	' .#....
		byte	%100000	' .....#
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%011100	' ..###.
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......
		byte	%000000	' ......


'**************************************************************************************************

{{
+------------------------------------------------------------------------------------------------------------------------------+
|				    TERMS OF USE: Parallax Object Exchange License					       |
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | |files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE	       |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}
